#include "gdt.h"
#include "../io/terminal.h"

struct sector gdt[GDT_SIZE];
struct gdt_location gdtp;

extern isr_wrapper;

void init_gdt()
{
    for(int i = 0; i < GDT_SIZE; i++)
    {
        gdt[i].lo_limit = 0;
        gdt[i].lo_offset = 0;
        gdt[i].mid_offset = 0;
        gdt[i].type = 0;
        gdt[i].granularity = 0;
        gdt[i].hi_offset = 0;
    }

    encode_gdt_entry(0, 0, 0, 0,0);
    encode_gdt_entry(1, 0, 0xffffffff, 0x9A, 0xCF);
    encode_gdt_entry(2, 0, 0xffffffff, 0x92, 0xCF);

    print_gdt_entry(0);
    print_gdt_entry(1);
    print_gdt_entry(2);

    construct_gdtp();
}

void construct_gdtp()
{
    gdtp.limit = sizeof(gdt);
    gdtp.offset = &gdt;
    set_gdtp();
}


void print_gdt_entry(int i)
{
    print("GDT ENTRY ");
    terminal_putint(i, 3);
    print("\n");
    terminal_putint((gdt[i].hi_offset << 16) + (gdt[i].mid_offset << 8) + gdt[i].lo_offset, 8);
    print(" ");
    terminal_putint(gdt[i].lo_limit, 5);
    print(" ");
    terminal_putint(gdt[i].type, 4);
    print("\nGDT ENTRY ");
    terminal_putint(i, 3);
    print(" FINISH\n");
}

void encode_gdt_entry(int i, uint32_t offset, uint16_t limit, uint8_t type, uint8_t granularity)
{
    gdt[i].lo_offset = (offset & 0xFFFF);
    gdt[i].mid_offset = (offset >> 16) & 0xFF;
    gdt[i].hi_offset = (offset >> 24) & 0xFF;

    gdt[i].lo_limit = (limit & 0xFFFF);
    gdt[i].granularity = ((limit >> 16) & 0x0F);

    gdt[i].granularity |= (granularity & 0xF0);
    gdt[i].type = type;
}