#include "idt.h"
#include "../io/terminal.h"


void interrupt_handler(void)
{
    terminal_writestring("INTERUPT CAUGHT!\n");
}
void hand_general_protection(void)
{
    terminal_writestring("GENERAL PROTECTION FAULT!\n");
}

struct interrupt_gate idt[IDT_SIZE];
struct idt_location idtp;

extern isr_wrapper;
extern int_general_protection;

void init_idt()
{
    for(int i = 0; i < IDT_SIZE; i++)
    {
        idt[i].lo_offset = 0;
        idt[i].selector = 0;
        idt[i].zero = 0;
        idt[i].type = 0;
        idt[i].hi_offset = 0;
    }

    for(int i = 0; i < IDT_SIZE; i++)
        encode_idt_entry(i, (unsigned)isr_wrapper, 0x08, 0x8E);
    encode_idt_entry(13, (unsigned)int_general_protection, 0x08, 0x8E);

    print_idt_entry(13);

    construct_idtp();
}

void construct_idtp()
{
    idtp.limit = sizeof(idt);
    idtp.offset = &idt;
    set_idtp();
}


void print_idt_entry(int i)
{
    print("IDT ENTRY ");
    terminal_putint(i, 3);
    print("\n");
    terminal_putint((idt[i].hi_offset << 16) + idt[i].lo_offset, 8);
    print(" ");
    terminal_putint(idt[i].selector, 5);
    print(" ");
    terminal_putint(idt[i].type, 4);
    print("\nIDT ENTRY ");
    terminal_putint(i, 3);
    print(" FINISH\n");
}

void encode_idt_entry(int i, unsigned long offset, uint16_t selector, uint8_t type_attr)
{
    idt[i].lo_offset = (offset & 0xFFFF);
    idt[i].selector = selector;
    idt[i].zero = 0;
    idt[i].type = type_attr;
    idt[i].hi_offset = offset >> 16;
}