#include "terminal.h"
/* Hardware text mode color constants. */
 
uint8_t make_color(enum vga_color fg, enum vga_color bg)
{
    return fg | bg << 4;
}
 
uint16_t make_vgaentry(char c, uint8_t color)
{
    uint16_t c16 = c;
    uint16_t color16 = color;
    return c16 | color16 << 8;
}
 
size_t strlen(const char* str)
{
    size_t ret = 0;
    while ( str[ret] != 0 )
        ret++;
    return ret;
}
 
static const size_t VGA_WIDTH = 80;
static const size_t VGA_HEIGHT = 25;
 
size_t terminal_row;
size_t terminal_column;
uint8_t terminal_color;
uint16_t* terminal_buffer;
 
void terminal_initialize()
{
    terminal_row = 0;
    terminal_column = 0;
    terminal_color = make_color(COLOR_LIGHT_GREY, COLOR_BLACK);
    terminal_buffer = (uint16_t*) 0xB8000;
    for ( size_t y = 0; y < VGA_HEIGHT; y++ )
    {
        for ( size_t x = 0; x < VGA_WIDTH; x++ )
        {
            const size_t index = y * VGA_WIDTH + x;
            terminal_buffer[index] = make_vgaentry(' ', terminal_color);
        }
    }
}
 
void terminal_setcolor(uint8_t color)
{
    terminal_color = color;
}
 
void terminal_putentryat(char c, uint8_t color, size_t x, size_t y)
{
    const size_t index = y * VGA_WIDTH + x;
    terminal_buffer[index] = make_vgaentry(c, color);
}

void push_terminal_up()
{
    for ( size_t y = 1; y < VGA_HEIGHT; y++ )
    {
        for ( size_t x = 0; x < VGA_WIDTH; x++ )
        {
            const size_t index = y * VGA_WIDTH + x;
            terminal_buffer[index + VGA_WIDTH] = terminal_buffer[index];
        }
    }
}
 
void terminal_putchar(char c)
{
    if (c == '\n')
    {
        terminal_column = 0;
        if ( ++terminal_row == VGA_HEIGHT )
        {
            push_terminal_up();
        }
    }
    else
    {
        terminal_putentryat(c, terminal_color, terminal_column, terminal_row);
        if ( ++terminal_column == VGA_WIDTH )
        {
            terminal_column = 0;
            if ( ++terminal_row == VGA_HEIGHT )
            {
                push_terminal_up();
            }
        }
    }
}
 
void terminal_putint(int in, int length)
{
    int out;
    char buff[10];
    int i;
    for (i = 0; i < length ; i++)
    {
        out = (in%10);
        buff[9-i] = out + '0';
        in/=10;
    }

    i--;
    for (;i >= 0;i--)
    {
        terminal_putchar(buff[9-i]);
    }
}
 
void terminal_writestring(const char* data)
{
    size_t datalen = strlen(data);
    for ( size_t i = 0; i < datalen; i++ )
        terminal_putchar(data[i]);
}

void print(const char* data)
{
    terminal_writestring(data);
}